from __future__ import annotations
import num_collect._cpp_module.num_collect_py_cpp_module
import typing
import datetime

__all__ = [
    "AlgorithmFailure",
    "AssertionFailure",
    "FileError",
    "InvalidArgument",
    "LogConfig",
    "LogLevel",
    "LogSinkBase",
    "LogTagConfig",
    "NumCollectCppException",
    "PreconditionNotSatisfied",
    "load_logging_config_file",
    "opt",
]

class NumCollectCppException(RuntimeError, Exception, BaseException):
    pass

class AssertionFailure(NumCollectCppException, RuntimeError, Exception, BaseException):
    pass

class FileError(NumCollectCppException, RuntimeError, Exception, BaseException):
    pass

class InvalidArgument(NumCollectCppException, RuntimeError, Exception, BaseException):
    pass

class LogConfig:
    """
    Class of configurations of logs.
    """

    @staticmethod
    def get_config_of(tag: str) -> LogTagConfig:
        """
        Get the configuration of a tag.

        Args:
            tag (str): Tag.

        Returns:
            LogTagConfig: Configuration.
        """
    @staticmethod
    def get_default_tag_config() -> LogTagConfig:
        """
        Get the default configuration of log tags.

        Returns:
            LogTagConfig: Default configuration of log tags.
        """
    @staticmethod
    def set_config_of(tag: str, config: LogTagConfig) -> None:
        """
        Set the configuration of a tag.

        Args:
            tag (str): Tag.
            config (LogTagConfig): Configuration.
        """
    @staticmethod
    def set_default_tag_config(config: LogTagConfig) -> None:
        """
        Set the default configuration of log tags.

        Args:
            config (LogTagConfig): Configuration.
        """
    pass

class LogLevel:
    """
    Enumeration of log levels.

    Members:

      trace : For internal trace logs.

      debug : For debug information. (Meant for use in user code, not in algorithms.)

      iteration : For logs of iterations.

      iteration_label : For labels of iteration logs.

      summary : For summary of calculations.

      info : For some information. (Meant for use in user code, not in algorithms.)

      warning : For warnings.

      error : For errors.

      critical : For critical errors.

      off : Turn off output (only for output log level).
    """

    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __init__(self, value: int) -> None: ...
    def __int__(self) -> int: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __repr__(self) -> str: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str:
        """
        :type: str
        """
    @property
    def value(self) -> int:
        """
        :type: int
        """
    __members__: dict  # value = {'trace': <LogLevel.trace: 0>, 'debug': <LogLevel.debug: 1>, 'iteration': <LogLevel.iteration: 2>, 'iteration_label': <LogLevel.iteration_label: 3>, 'summary': <LogLevel.summary: 4>, 'info': <LogLevel.info: 5>, 'warning': <LogLevel.warning: 6>, 'error': <LogLevel.error: 7>, 'critical': <LogLevel.critical: 8>, 'off': <LogLevel.off: 9>}
    critical: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.critical: 8>
    debug: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.debug: 1>
    error: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.error: 7>
    info: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.info: 5>
    iteration: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.iteration: 2>
    iteration_label: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.iteration_label: 3>
    off: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.off: 9>
    summary: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.summary: 4>
    trace: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.trace: 0>
    warning: num_collect._cpp_module.num_collect_py_cpp_module.LogLevel  # value = <LogLevel.warning: 6>
    pass

class LogSinkBase:
    """
    Interface of log sinks.
    """

    def __init__(self) -> None: ...
    def write(
        self,
        time: datetime.datetime,
        tag: str,
        level: LogLevel,
        file_path: str,
        line: int,
        column: int,
        function_name: str,
        body: str,
    ) -> None:
        """
        Write a log.

        Args:
            time (datetime.datetime): Time.
            tag (str): Tag.
            level (LogLevel): Log level.
            file_path (str): File path.
            line (int): Line number in the file.
            column (int): Column number in the line.
            function_name (str): Function name.
            body (str): Log body.
        """
    pass

class LogTagConfig:
    """
    Class to hold configurations for log tags.
    """

    def __init__(self) -> None: ...
    @property
    def iteration_label_period(self) -> int:
        """
        Period to write labels of iteration logs.

        :type: int
        """
    @iteration_label_period.setter
    def iteration_label_period(self, arg1: int) -> None:
        """
        Period to write labels of iteration logs.
        """
    @property
    def iteration_output_period(self) -> int:
        """
        Period to write iteration logs.

        :type: int
        """
    @iteration_output_period.setter
    def iteration_output_period(self, arg1: int) -> None:
        """
        Period to write iteration logs.
        """
    @property
    def output_log_level(self) -> LogLevel:
        """
        Minimum log level to output.

        :type: LogLevel
        """
    @output_log_level.setter
    def output_log_level(self, arg1: LogLevel) -> None:
        """
        Minimum log level to output.
        """
    @property
    def output_log_level_in_child_iterations(self) -> LogLevel:
        """
        Minimum log level to output in child iterations.

        :type: LogLevel
        """
    @output_log_level_in_child_iterations.setter
    def output_log_level_in_child_iterations(self, arg1: LogLevel) -> None:
        """
        Minimum log level to output in child iterations.
        """
    @property
    def sink(self) -> LogSinkBase:
        """
        Log sink.

        :type: LogSinkBase
        """
    @sink.setter
    def sink(self, arg1: LogSinkBase) -> None:
        """
        Log sink.
        """
    pass

class AlgorithmFailure(NumCollectCppException, RuntimeError, Exception, BaseException):
    pass

class PreconditionNotSatisfied(
    NumCollectCppException, RuntimeError, Exception, BaseException
):
    pass

def load_logging_config_file(file_path: str) -> None:
    """
    Parse and apply configurations of logging from a file.

    Args:
        file_path (str): File path.
    """



"""Module of loggers."""

import datetime
import inspect
import os
import types
import typing

from num_collect._cpp_module.num_collect_py_cpp_module import (
    LogConfig,
    LogLevel,
    LogTagConfig,
)

# For generation of source code information,
# I referred https://github.com/python/cpython/blob/c2bd55d26f8eb2850eb9f9026b5d7f0ed1420b65/Lib/logging/__init__.py.


def tester() -> None:
    """Function for THIS_FILEPATH."""


THIS_FILEPATH = os.path.normcase(tester.__code__.co_filename)


def is_in_this_file(frame: types.FrameType) -> bool:
    """Check whether a frame is in this file.

    Args:
        frame (types.FrameType): Frame.

    Returns:
        bool: Whether the frame is in this file.
    """
    return os.path.normcase(frame.f_code.co_filename) == THIS_FILEPATH


def get_source_info() -> typing.Tuple[str, int, str]:
    """Get the information of the source code for logs.

    Returns:
        typing.Tuple[str, int, str]: File path, line number, and function name.
    """
    frame = inspect.currentframe()

    while True:
        if frame is None:
            file_path = ""
            line = 0
            function_name = ""
            return (file_path, line, function_name)

        if not is_in_this_file(frame):
            break

        frame = frame.f_back

    file_path = frame.f_code.co_filename
    line = frame.f_lineno
    function_name = frame.f_code.co_name
    return (file_path, line, function_name)


class NumCollectLogger:
    """Class of loggers to use logging module in C++.

    Args:
        tag (str): Tag.

    """

    def __init__(
        self,
        tag: typing.Optional[str] = None,
        *,
        config: typing.Optional[LogTagConfig] = None,
    ) -> None:
        if not tag:
            tag = ""
        if not config:
            config = LogConfig.get_config_of(tag)

        self._tag = tag
        self._config = config
        self._sink = self._config.sink

    def log(self, level: LogLevel, message: str, *args: typing.Any) -> None:
        """Write a log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            level (LogLevel): Log level.
            message (str): Message.
        """
        if level < self._config.output_log_level:
            return

        if args:
            body = message % args
        else:
            body = message

        time = datetime.datetime.now()
        file_path, line, function_name = get_source_info()
        column = 0  # Unknown.

        self._sink.write(
            time=time,
            tag=self._tag,
            level=level,
            file_path=file_path,
            line=line,
            column=column,
            function_name=function_name,
            body=body,
        )

    def trace(self, message: str, *args: typing.Any) -> None:
        """Write a trace log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.trace, message, *args)

    def debug(self, message: str, *args: typing.Any) -> None:
        """Write a debug log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.debug, message, *args)

    def iteration(self, message: str, *args: typing.Any) -> None:
        """Write a iteration log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.iteration, message, *args)

    def iteration_label(self, message: str, *args: typing.Any) -> None:
        """Write a iteration_label log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.iteration_label, message, *args)

    def summary(self, message: str, *args: typing.Any) -> None:
        """Write a summary log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.summary, message, *args)

    def info(self, message: str, *args: typing.Any) -> None:
        """Write a info log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.info, message, *args)

    def warning(self, message: str, *args: typing.Any) -> None:
        """Write a warning log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.warning, message, *args)

    def error(self, message: str, *args: typing.Any) -> None:
        """Write a error log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.error, message, *args)

    def critical(self, message: str, *args: typing.Any) -> None:
        """Write a critical log.

        Additional arguments are used for formatting the message
        as in logging.Logger in Python standard library.

        Args:
            message (str): Message.
        """
        self.log(LogLevel.critical, message, *args)



from __future__ import annotations
import num_collect._cpp_module.num_collect_py_cpp_module.opt.multi_variate
import typing
import numpy

__all__ = [
    "AdaptiveDiagonalCurves",
    "DividingRectangles",
    "DownhillSimplex",
    "ObjectiveFunctionBase",
]

class AdaptiveDiagonalCurves:
    """
    Class of adaptive diagonal curves (ADC) method :cite:`Sergeyev2006` for optimization.

    Args:
        obj_fun (num_collect.opt.multi_variate.ObjectiveFunctionBase): Objective function.
    """

    def __init__(self, obj_fun: ObjectiveFunctionBase) -> None: ...
    def decrease_rate_bound(self, value: float) -> AdaptiveDiagonalCurves:
        """
        Set the rate of function value used to check whether the function value decreased in the current phase.

        Args:
            value (float): Value.
        """
    def init(self, lower: numpy.ndarray, upper: numpy.ndarray) -> None:
        """
        Initialize the algorithm.

        Args:
            lower (numpy.ndarray): Lower limit of the range of variables.
            upper (numpy.ndarray): Upper limit of the range of variables.
        """
    def is_stop_criteria_satisfied(self) -> None:
        """
        Determine if stopping criteria of the algorithm are satisfied.

        Returns:
            bool: True if stopping criteria of the algorithm are satisfied.
        """
    def iterate(self) -> None:
        """
        Iterate once.

        Note:
            Any required initializations (with init functions) are
            assumed to have been done.
        """
    def max_evaluations(self, value: int) -> AdaptiveDiagonalCurves:
        """
        Set the maximum number of function evaluations.

        Args:
            value (int): Value.
        """
    def min_rate_imp(self, value: float) -> AdaptiveDiagonalCurves:
        """
        Set the minimum rate of improvement in the function value required for potentially optimal rectangles.

        Args:
            value (float): Value.
        """
    def solve(self) -> None:
        """
        Solve the problem.

        Iterate the algorithm until the stopping criteria are satisfied.

        Note:
            Any required initializations (with init functions) are
            assumed to have been done.
        """
    @property
    def evaluations(self) -> int:
        """
        Number of function evaluations.

        :type: int
        """
    @property
    def iterations(self) -> int:
        """
        Number of iterations.

        :type: int
        """
    @property
    def opt_value(self) -> float:
        """
        Current optimal value.

        :type: float
        """
    @property
    def opt_variable(self) -> numpy.ndarray:
        """
        Current optimal variable.

        :type: numpy.ndarray
        """
    pass

class DividingRectangles:
    """
    Class of dividing rectangles (DIRECT) method :cite:`Jones1993` for optimization.

    Args:
        obj_fun (num_collect.opt.multi_variate.ObjectiveFunctionBase): Objective function.
    """

    def __init__(self, obj_fun: ObjectiveFunctionBase) -> None: ...
    def init(self, lower: numpy.ndarray, upper: numpy.ndarray) -> None:
        """
        Initialize the algorithm.

        Args:
            lower (numpy.ndarray): Lower limit of the range of variables.
            upper (numpy.ndarray): Upper limit of the range of variables.
        """
    def is_stop_criteria_satisfied(self) -> None:
        """
        Determine if stopping criteria of the algorithm are satisfied.

        Returns:
            bool: True if stopping criteria of the algorithm are satisfied.
        """
    def iterate(self) -> None:
        """
        Iterate once.

        Note:
            Any required initializations (with init functions) are
            assumed to have been done.
        """
    def max_evaluations(self, value: int) -> DividingRectangles:
        """
        Set the maximum number of function evaluations.

        Args:
            value (int): Value.
        """
    def min_rate_imp(self, value: float) -> DividingRectangles:
        """
        Set the minimum rate of improvement in the function value required for potentially optimal rectangles.

        Args:
            value (float): Value.
        """
    def solve(self) -> None:
        """
        Solve the problem.

        Iterate the algorithm until the stopping criteria are satisfied.

        Note:
            Any required initializations (with init functions) are
            assumed to have been done.
        """
    @property
    def evaluations(self) -> int:
        """
        Number of function evaluations.

        :type: int
        """
    @property
    def iterations(self) -> int:
        """
        Number of iterations.

        :type: int
        """
    @property
    def opt_value(self) -> float:
        """
        Current optimal value.

        :type: float
        """
    @property
    def opt_variable(self) -> numpy.ndarray:
        """
        Current optimal variable.

        :type: numpy.ndarray
        """
    pass

class DownhillSimplex:
    """
    Class of downhill simplex method :cite:`Press2007`.

    Args:
        obj_fun (num_collect.opt.multi_variate.ObjectiveFunctionBase): Objective function.
    """

    def __init__(self, obj_fun: ObjectiveFunctionBase) -> None: ...
    def init(self, init_var: numpy.ndarray, *, width: float = 0.1) -> None:
        """
        Initialize the algorithm.

        Args:
            init_var (numpy.ndarray): Initial variable.
            width (float, optional): Width of the initial simplex.
        """
    def is_stop_criteria_satisfied(self) -> None:
        """
        Determine if stopping criteria of the algorithm are satisfied.

        Returns:
            bool: True if stopping criteria of the algorithm are satisfied.
        """
    def iterate(self) -> None:
        """
        Iterate once.

        Note:
            Any required initializations (with init functions) are
            assumed to have been done.
        """
    def solve(self) -> None:
        """
        Solve the problem.

        Iterate the algorithm until the stopping criteria are satisfied.

        Note:
            Any required initializations (with init functions) are
            assumed to have been done.
        """
    def tol_simplex_size(self, value: float) -> DownhillSimplex:
        """
        Set tolerance of size of simplex.

        Args:
            value (float): Value.
        """
    @property
    def evaluations(self) -> int:
        """
        Number of function evaluations.

        :type: int
        """
    @property
    def iterations(self) -> int:
        """
        Number of iterations.

        :type: int
        """
    @property
    def opt_value(self) -> float:
        """
        Current optimal value.

        :type: float
        """
    @property
    def opt_variable(self) -> numpy.ndarray:
        """
        Current optimal variable.

        :type: numpy.ndarray
        """
    pass

class ObjectiveFunctionBase:
    """
    Base class of multi-variate objective functions.

    Inherit objective functions from this class and
    implement evaluate_on function.
    """

    def __init__(self) -> None: ...
    def evaluate_on(self, var: numpy.ndarray) -> None:
        """
        Evaluate this objective function on a variable.

        Args:
            var (numpy.ndarray): Variable.
        """
    @property
    def value(self) -> float:
        """
        Function value.

        :type: float
        """
    @value.setter
    def value(self, arg1: float) -> None:
        """
        Function value.
        """
    pass



"""Module of basic classes in num_collect package."""

from num_collect._cpp_module.num_collect_py_cpp_module import (
    AlgorithmFailure,
    AssertionFailure,
    FileError,
    InvalidArgument,
    NumCollectCppException,
    PreconditionNotSatisfied,
)

all_exports: list = [
    NumCollectCppException,
    AssertionFailure,
    PreconditionNotSatisfied,
    InvalidArgument,
    AlgorithmFailure,
    FileError,
]
for e in all_exports:
    e.__module__ == __name__

__all__ = [e.__name__ for e in all_exports]

del all_exports
del e



"""Build C++ module for poetry."""

import os
import pathlib
import subprocess
import sys
import typing

import setuptools
import setuptools.command.build_ext
import toml

THIS_DIR = pathlib.Path(__file__).absolute().parent


class CMakeExtension(setuptools.Extension):
    """CMake extensions."""

    def __init__(self, name: str) -> None:
        super().__init__(name=name, sources=[])


class ExtensionBuilder(setuptools.command.build_ext.build_ext):
    """Builder of extensions."""

    def build_extension(self, ext: setuptools.Extension) -> None:
        if not isinstance(ext, CMakeExtension):
            super().build_extension(ext)

        lib_root_dir = (
            pathlib.Path.cwd() / self.get_ext_fullpath("num_collect")
        ).parent.resolve()

        source_dir = str(THIS_DIR)
        build_dir = f"{self.build_temp}/cpp_module"
        build_type = "Release"

        os.makedirs(build_dir, exist_ok=True)

        # Add Conan remotes.
        subprocess.run(
            [
                "conan",
                "remote",
                "add",
                "-f",
                "cpp-hash-tables",
                "https://gitlab.com/api/v4/projects/35726343/packages/conan",
            ],
            check=True,
        )
        subprocess.run(
            [
                "conan",
                "remote",
                "add",
                "-f",
                "numerical-collection-cpp",
                "https://gitlab.com/api/v4/projects/25109105/packages/conan",
            ],
            check=True,
        )

        # Install Conan packages.
        subprocess.run(
            [
                "conan",
                "install",
                "--build",
                "missing",
                "-s",
                f"build_type={build_type}",
                source_dir,
            ],
            cwd=build_dir,
            check=True,
        )

        # Build.
        subprocess.run(
            [
                "cmake",
                # cspell: ignore DBUILD, DPYTHON, DNUM
                f"-DCMAKE_BUILD_TYPE={build_type}",
                f"-DPYTHON_EXECUTABLE={sys.executable}",
                f"-DNUM_COLLECT_PY_ROOT={lib_root_dir}",
                source_dir,
            ],
            cwd=build_dir,
            check=True,
        )
        subprocess.run(
            [
                "cmake",
                "--build",
                ".",
                "--config",
                build_type,
            ],
            cwd=build_dir,
            check=True,
        )


def read_file(filepath: str) -> str:
    """Read all lines in a file."""
    with open(filepath, mode="r", encoding="utf8") as file:
        return file.read()


def dir_path_to_package_path(dir_path: pathlib.Path) -> str:
    """Convert directory path to package path."""
    return dir_path.relative_to(THIS_DIR).as_posix().replace("/", ".")


def collect_packages_in(dir_path: pathlib.Path) -> typing.List[str]:
    """Collect packages in a directory."""
    if (
        not (dir_path / "__init__.py").exists()
        and not (dir_path / "__init__.pyi").exists()
    ):
        return []

    res = [dir_path_to_package_path(dir_path)]
    for elem in dir_path.iterdir():
        if elem.is_dir():
            res = res + collect_packages_in(elem)

    return res


def collect_packages() -> typing.List[str]:
    """Collect packages."""
    return collect_packages_in(THIS_DIR / "num_collect")


pyproject_toml = toml.load(str(THIS_DIR / "pyproject.toml"))
pyproject_toml_tool_poetry = pyproject_toml["tool"]["poetry"]

package_list = collect_packages()
package_data = {name: ["py.typed"] for name in package_list}
package_data.update(
    {
        name: ["py.typed", "__init__.pyi"]
        for name in package_list
        if name.startswith("num_collect._cpp_module.num_collect_py_cpp_module")
    }
)
package_data.update(
    {"num_collect._cpp_module": ["py.typed", "num_collect_py_cpp_module.*"]}
)

# Reference of keywords:
# https://setuptools.pypa.io/en/latest/references/keywords.html
setuptools.setup(
    name=pyproject_toml_tool_poetry["name"],
    version=pyproject_toml_tool_poetry["version"],
    description=pyproject_toml_tool_poetry["description"],
    long_description=read_file(str(THIS_DIR / "README.md")),
    long_description_content_type="text/markdown",
    author="Kenta Kabashima",
    author_email="kenta_program37@hotmail.co.jp",
    url=pyproject_toml_tool_poetry["homepage"],
    classifiers=pyproject_toml_tool_poetry["classifiers"],
    license=pyproject_toml_tool_poetry["license"],
    ext_modules=[
        CMakeExtension(
            name="num_collect._cpp_module.num_collect_py_cpp_module",
        )
    ],
    cmdclass={"build_ext": ExtensionBuilder},  # cspell: disable-line
    zip_safe=False,
    packages=package_list,
    include_package_data=False,
    package_data=package_data,
    python_requires=pyproject_toml["tool"]["poetry"]["dependencies"]["python"],
    install_requires=["numpy>=1.24.2,<2.0"],
    project_urls={
        "Bug Tracker": "https://gitlab.com/MusicScience37Projects/numerical-analysis/numerical-collection-py/-/issues",
        "Documentation": pyproject_toml_tool_poetry["documentation"],
        "Source Code": pyproject_toml_tool_poetry["repository"],
    },
)



"""Module of logging in num_collect package."""

from num_collect._cpp_module.num_collect_py_cpp_module import (
    LogConfig,
    LogLevel,
    LogSinkBase,
    LogTagConfig,
    load_logging_config_file,
)
from num_collect.logging._handler import NumCollectLogHandler
from num_collect.logging._logger import NumCollectLogger

all_exports: list = [
    LogLevel,
    LogSinkBase,
    LogTagConfig,
    LogConfig,
    NumCollectLogger,
    NumCollectLogHandler,
    load_logging_config_file,
]
for e in all_exports:
    e.__module__ == __name__

__all__ = [e.__name__ for e in all_exports]

del all_exports
del e
